﻿// --------------------------------------------------------------------------------------------
// Stefan Cruysberghs
// http://www.scip.be
// Article: Customize WCF RIA Services code generation
// http://www.scip.be/index.php?Page=ArticlesNET36
// --------------------------------------------------------------------------------------------
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using Microsoft.ServiceModel.DomainServices.Tools.TextTemplate.CSharpGenerators;
using ScipBe.Demo.RiaCodeGen.Web.Domain;

namespace ScipBe.Demo.RiaCodeGen.Web.Generators
{
    public class MyEntityGenerator : CSharpEntityGenerator
    {
        XmlCommentsProvider xmlCommentsProvider;

        public override string TransformText()
        {
            var myEntity = "MyEntity";
            if (typeof(IId).IsAssignableFrom(Type))
            {
                myEntity += ", IId";
            }

            var generatedCode = base.TransformText();

            var debuggerDisplayAttributes = Type.GetCustomAttributes(typeof(DebuggerDisplayAttribute), true)
                .Cast<DebuggerDisplayAttribute>().FirstOrDefault();

            if (debuggerDisplayAttributes != null)
            {
                var displayName = debuggerDisplayAttributes.Value;
                generatedCode = generatedCode
                    .Replace("public sealed partial class", 
                    "[DebuggerDisplay(\"" + displayName + "\")]\r\n    public sealed partial class");
            }

            generatedCode = generatedCode
                .Replace("System.ServiceModel.DomainServices.Client.Entity", myEntity);

            generatedCode = generatedCode
                .Replace("[System.ComponentModel.DataAnnotations.RequiredAttribute()]", "[MyRequired]");

            return generatedCode;
        }

        protected override void GenerateClassDeclaration()
        {
            WriteLine("using ScipBe.Demo.RiaCodeGen.Framework;");
            WriteLine("using ScipBe.Demo.RiaCodeGen.Web.Domain;");
            WriteLine("using System.Diagnostics;");

            // Add XML documentation for class
            xmlCommentsProvider = new XmlCommentsProvider(this.Type, this.ClientCodeGenerator.Options.ClientProjectPath);
            WriteLine(xmlCommentsProvider.GetClassComments(this.Type));

            base.GenerateClassDeclaration();
        }

        protected override void GenerateProperty(PropertyDescriptor propertyDescriptor)
        {
            // Add XML documentation for property
            if (xmlCommentsProvider != null)
            {
                var comments = xmlCommentsProvider.GetPropertyComments(propertyDescriptor);
                if (!string.IsNullOrEmpty(comments))
                {
                    WriteLine(comments);
                }
            }

            base.GenerateProperty(propertyDescriptor);
        }
    }
}